@@ -1,11 +1,15 @@
 #!/usr/bin/python3
 
 import logging
+import os
 import struct
 import sys
 
+from fontTools import ttLib
 from lxml import etree
 
+from keyman_config.kmpmetadata import parsemetadata
+
 # .kvk file format
 # KVK files are variable length files with variable sized structures.
 
@@ -311,7 +315,18 @@ def get_modifer(key):
     return modifier
 
 
-def convert_ldml(kvkData):
+def _fontFacename(fontFilename):
+    """Get the facename from fontFilename's TTF tables"""
+    # From https://github.com/mcfletch/ttfquery/blob/master/ttfquery/describe.py
+    FONT_SPECIFIER_NAME_ID = 4
+    font = ttLib.TTFont(fontFilename)
+    for record in font['name'].names:
+        if record.nameID == FONT_SPECIFIER_NAME_ID:
+            return record.toUnicode()
+    return None
+
+
+def convert_ldml(keyboardName, kvkData, kmpJsonFilename):
     keymaps = {}
 
     for key in kvkData.Keys:
@@ -349,7 +364,19 @@ def convert_ldml(kvkData):
             else:
                 keymaps["shift"] = (uskey,)
 
+    info, system, options, keyboards, files = parsemetadata(kmpJsonFilename)
+
     ldml = etree.Element("keyboard", locale="zzz-keyman")
+    for keyboard in keyboards:
+        if keyboard['id'] != keyboardName:
+            continue
+        if 'oskFont' in keyboard:
+            fontFile = os.path.join(os.path.dirname(kmpJsonFilename), keyboard['oskFont'])
+            font = _fontFacename(fontFile)
+            if font:
+                ldml.set('keymanFacename', font)
+        break
+
     etree.SubElement(ldml, "version", platform="11")
     names = etree.SubElement(ldml, "names")
     names.append(etree.Element("name", value="ZZZ"))
@@ -402,6 +429,7 @@ def parse_kvk_file(kvkfile):
     return kvkData
 
 
-def convert_kvk_to_ldml(kvkfile):
+def convert_kvk_to_ldml(name, kvkfile):
     kvkData = parse_kvk_file(kvkfile)
-    return convert_ldml(kvkData)
+    kmpJsonFilename = os.path.join(os.path.dirname(kvkfile), 'kmp.json')
+    return convert_ldml(name, kvkData, kmpJsonFilename)
